@@ -308,6 +308,9 @@ def from_attributes_callback(ctx: MethodContext) -> Type:
     pydantic_metadata = model_type.type.metadata.get(METADATA_KEY)
     if pydantic_metadata is None:
         return ctx.default_return_type
+    if not any(base.fullname == BASEMODEL_FULLNAME for base in model_type.type.mro):
+        # not a Pydantic v2 model
+        return ctx.default_return_type
     from_attributes = pydantic_metadata.get('config', {}).get('from_attributes')
     if from_attributes is not True:
         error_from_attributes(model_type.type.name, ctx.api, ctx.context)
@@ -321,8 +324,10 @@ class PydanticModelField:
         self,
         name: str,
         alias: str | None,
+        is_frozen: bool,
         has_dynamic_alias: bool,
         has_default: bool,
+        strict: bool | None,
         line: int,
         column: int,
         type: Type | None,
@@ -330,8 +335,10 @@ class PydanticModelField:
     ):
         self.name = name
         self.alias = alias
+        self.is_frozen = is_frozen
         self.has_dynamic_alias = has_dynamic_alias
         self.has_default = has_default
+        self.strict = strict
         self.line = line
         self.column = column
         self.type = type
@@ -341,19 +348,29 @@ class PydanticModelField:
         self,
         current_info: TypeInfo,
         typed: bool,
+        model_strict: bool,
         force_optional: bool,
         use_alias: bool,
         api: SemanticAnalyzerPluginInterface,
         force_typevars_invariant: bool,
+        is_root_model_root: bool,
     ) -> Argument:
         """Based on mypy.plugins.dataclasses.DataclassAttribute.to_argument."""
         variable = self.to_var(current_info, api, use_alias, force_typevars_invariant)
-        type_annotation = self.expand_type(current_info, api) if typed else AnyType(TypeOfAny.explicit)
+
+        strict = model_strict if self.strict is None else self.strict
+        if typed or strict:
+            type_annotation = self.expand_type(current_info, api)
+        else:
+            type_annotation = AnyType(TypeOfAny.explicit)
+
         return Argument(
             variable=variable,
             type_annotation=type_annotation,
             initializer=None,
-            kind=ARG_NAMED_OPT if force_optional or self.has_default else ARG_NAMED,
+            kind=ARG_OPT
+            if is_root_model_root
+            else (ARG_NAMED_OPT if force_optional or self.has_default else ARG_NAMED),
         )
 
     def expand_type(
@@ -406,8 +423,10 @@ class PydanticModelField:
         return {
             'name': self.name,
             'alias': self.alias,
+            'is_frozen': self.is_frozen,
             'has_dynamic_alias': self.has_dynamic_alias,
             'has_default': self.has_default,
+            'strict': self.strict,
             'line': self.line,
             'column': self.column,
             'type': self.type.serialize(),
@@ -467,6 +486,7 @@ class PydanticModelTransformer:
         'from_attributes',
         'populate_by_name',
         'alias_generator',
+        'strict',
     }
 
     def __init__(
@@ -505,8 +525,7 @@ class PydanticModelTransformer:
 
         is_settings = any(base.fullname == BASESETTINGS_FULLNAME for base in info.mro[:-1])
         self.add_initializer(fields, config, is_settings, is_root_model)
-        if not is_root_model:
-            self.add_model_construct_method(fields, config, is_settings)
+        self.add_model_construct_method(fields, config, is_settings, is_root_model)
         self.set_frozen(fields, self._api, frozen=config.frozen is True)
 
         self.adjust_decorator_signatures()
@@ -527,7 +546,7 @@ class PydanticModelTransformer:
         Teach mypy this by marking any function whose outermost decorator is a `validator()`,
         `field_validator()` or `serializer()` call as a `classmethod`.
         """
-        for name, sym in self._cls.info.names.items():
+        for sym in self._cls.info.names.values():
             if isinstance(sym.node, Decorator):
                 first_dec = sym.node.original_decorators[0]
                 if (
@@ -791,6 +810,7 @@ class PydanticModelTransformer:
             )
 
         has_default = self.get_has_default(stmt)
+        strict = self.get_strict(stmt)
 
         if sym.type is None and node.is_final and node.is_inferred:
             # This follows the logic from the dataclasses plugin. The following comment is taken verbatim:
@@ -813,13 +833,16 @@ class PydanticModelTransformer:
         alias, has_dynamic_alias = self.get_alias_info(stmt)
         if has_dynamic_alias and not model_config.populate_by_name and self.plugin_config.warn_required_dynamic_aliases:
             error_required_dynamic_aliases(self._api, stmt)
+        is_frozen = self.is_field_frozen(stmt)
 
         init_type = self._infer_dataclass_attr_init_type(sym, lhs.name, stmt)
         return PydanticModelField(
             name=lhs.name,
             has_dynamic_alias=has_dynamic_alias,
             has_default=has_default,
+            strict=strict,
             alias=alias,
+            is_frozen=is_frozen,
             line=stmt.line,
             column=stmt.column,
             type=init_type,
@@ -875,14 +898,17 @@ class PydanticModelTransformer:
             return  # Don't generate an __init__ if one already exists
 
         typed = self.plugin_config.init_typed
+        model_strict = bool(config.strict)
         use_alias = config.populate_by_name is not True
         requires_dynamic_aliases = bool(config.has_alias_generator and not config.populate_by_name)
         args = self.get_field_arguments(
             fields,
             typed=typed,
+            model_strict=model_strict,
             requires_dynamic_aliases=requires_dynamic_aliases,
             use_alias=use_alias,
             is_settings=is_settings,
+            is_root_model=is_root_model,
             force_typevars_invariant=True,
         )
 
@@ -911,7 +937,11 @@ class PydanticModelTransformer:
         add_method(self._api, self._cls, '__init__', args=args, return_type=NoneType())
 
     def add_model_construct_method(
-        self, fields: list[PydanticModelField], config: ModelConfigData, is_settings: bool
+        self,
+        fields: list[PydanticModelField],
+        config: ModelConfigData,
+        is_settings: bool,
+        is_root_model: bool,
     ) -> None:
         """Adds a fully typed `model_construct` classmethod to the class.
 
@@ -923,13 +953,19 @@ class PydanticModelTransformer:
         fields_set_argument = Argument(Var('_fields_set', optional_set_str), optional_set_str, None, ARG_OPT)
         with state.strict_optional_set(self._api.options.strict_optional):
             args = self.get_field_arguments(
-                fields, typed=True, requires_dynamic_aliases=False, use_alias=False, is_settings=is_settings
+                fields,
+                typed=True,
+                model_strict=bool(config.strict),
+                requires_dynamic_aliases=False,
+                use_alias=False,
+                is_settings=is_settings,
+                is_root_model=is_root_model,
             )
         if not self.should_init_forbid_extra(fields, config):
             var = Var('kwargs')
             args.append(Argument(var, AnyType(TypeOfAny.explicit), None, ARG_STAR2))
 
-        args = [fields_set_argument] + args
+        args = args + [fields_set_argument] if is_root_model else [fields_set_argument] + args
 
         add_method(
             self._api,
@@ -951,7 +987,7 @@ class PydanticModelTransformer:
             if sym_node is not None:
                 var = sym_node.node
                 if isinstance(var, Var):
-                    var.is_property = frozen
+                    var.is_property = frozen or field.is_frozen
                 elif isinstance(var, PlaceholderNode) and not self._api.final_iteration:
                     # See https://github.com/pydantic/pydantic/issues/5191 to hit this branch for test coverage
                     self._api.defer()
@@ -1028,6 +1064,22 @@ class PydanticModelTransformer:
         # Has no default if the "default value" is Ellipsis (i.e., `field_name: Annotation = ...`)
         return not isinstance(expr, EllipsisExpr)
 
+    @staticmethod
+    def get_strict(stmt: AssignmentStmt) -> bool | None:
+        """Returns a the `strict` value of a field if defined, otherwise `None`."""
+        expr = stmt.rvalue
+        if isinstance(expr, CallExpr) and isinstance(expr.callee, RefExpr) and expr.callee.fullname == FIELD_FULLNAME:
+            for arg, name in zip(expr.args, expr.arg_names):
+                if name != 'strict':
+                    continue
+                if isinstance(arg, NameExpr):
+                    if arg.fullname == 'builtins.True':
+                        return True
+                    elif arg.fullname == 'builtins.False':
+                        return False
+                return None
+        return None
+
     @staticmethod
     def get_alias_info(stmt: AssignmentStmt) -> tuple[str | None, bool]:
         """Returns a pair (alias, has_dynamic_alias), extracted from the declaration of the field defined in `stmt`.
@@ -1056,13 +1108,40 @@ class PydanticModelTransformer:
                 return None, True
         return None, False
 
+    @staticmethod
+    def is_field_frozen(stmt: AssignmentStmt) -> bool:
+        """Returns whether the field is frozen, extracted from the declaration of the field defined in `stmt`.
+
+        Note that this is only whether the field was declared to be frozen in a `<field_name> = Field(frozen=True)`
+        sense; this does not determine whether the field is frozen because the entire model is frozen; that is
+        handled separately.
+        """
+        expr = stmt.rvalue
+        if isinstance(expr, TempNode):
+            # TempNode means annotation-only
+            return False
+
+        if not (
+            isinstance(expr, CallExpr) and isinstance(expr.callee, RefExpr) and expr.callee.fullname == FIELD_FULLNAME
+        ):
+            # Assigned value is not a call to pydantic.fields.Field
+            return False
+
+        for i, arg_name in enumerate(expr.arg_names):
+            if arg_name == 'frozen':
+                arg = expr.args[i]
+                return isinstance(arg, NameExpr) and arg.fullname == 'builtins.True'
+        return False
+
     def get_field_arguments(
         self,
         fields: list[PydanticModelField],
         typed: bool,
+        model_strict: bool,
         use_alias: bool,
         requires_dynamic_aliases: bool,
         is_settings: bool,
+        is_root_model: bool,
         force_typevars_invariant: bool = False,
     ) -> list[Argument]:
         """Helper function used during the construction of the `__init__` and `model_construct` method signatures.
@@ -1074,10 +1153,12 @@ class PydanticModelTransformer:
             field.to_argument(
                 info,
                 typed=typed,
+                model_strict=model_strict,
                 force_optional=requires_dynamic_aliases or is_settings,
                 use_alias=use_alias,
                 api=self._api,
                 force_typevars_invariant=force_typevars_invariant,
+                is_root_model_root=is_root_model and field.name == 'root',
             )
             for field in fields
             if not (use_alias and field.has_dynamic_alias)
@@ -1122,12 +1203,14 @@ class ModelConfigData:
         from_attributes: bool | None = None,
         populate_by_name: bool | None = None,
         has_alias_generator: bool | None = None,
+        strict: bool | None = None,
     ):
         self.forbid_extra = forbid_extra
         self.frozen = frozen
         self.from_attributes = from_attributes
         self.populate_by_name = populate_by_name
         self.has_alias_generator = has_alias_generator
+        self.strict = strict
 
     def get_values_dict(self) -> dict[str, Any]:
         """Returns a dict of Pydantic model config names to their values.
